function aligned_images = align_images(images, reference)
% function aligned_images = align_images(images, reference)
%
% Given one or more peak pressure images, align them to the
% provided reference image using pairwise image registration.
%
% Inputs:
%    - images: This can either be (a) a single peak pressure image
%              provided in matrix form, or (b) an Nx1 cell array
%              containing N peak pressure images.
%    - reference: The peak pressure image that all other given
%                 images will be aligned to (e.g. the template or
%                 the predicted peak pressure image).
%
% Outputs: 
%    - aligned_images: The inputted images after being aligned to
%                      the provided template. The format of this
%                      output matches the 'images' input.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>
%

  if ~exist('scalar_img_register'),
      p = mfilename('fullpath');
      [dir,~,~] = fileparts(p);
      addpath([dir, filesep, 'helper_functions']);
  end

  fprintf('Aligning Images...');
  if iscell(images),
      aligned_images = cell(size(images));
      for ii = 1:numel(images),
          aligned_images{ii} = register(reference, images{ii});
          fprintf('.');
      end
  else
      aligned_images = register(reference, images);
  end
  
  fprintf('Done!\n');
  
  
% Helper function to register a pair of images
function registered_image = register(template, img);

  [~, T, D] = scalar_img_register(template, img, 'deformable');
  tmp_img = my_img_warp(img, T, D);
  pad_sz = size(template) - size(tmp_img);
  tmp_img = padarray(tmp_img, max(0, pad_sz), 'post');
  registered_image = tmp_img(1:size(template,1), ...
                             1:size(template,2));
  