function models = build_models(aligned_images, demographics)
% function models = build_models(aligned_images, demographics)
%
% Given a set of aligned peak pressure images from N healthy
% individuals, and the demographics from those same N individuals,
% build the PAPPI statistical models. 
%
% Inputs:
%    - aligned_images: An Nx1 cell array containing the aligned
%                      peak pressure images from a group of healthy
%                      individuals.
%    - demographics: A 5xN matrix containing the demographics of
%                    our healthy individuals in the following
%                    order:
%                     * age (in years)
%                     * sex (0 for female, 1 for male)
%                     * weight (in kilograms)
%                     * height (in centimeters)
%                     * shoe size (in the EU scale)
%
% Outputs:
%    - models: A MATLAB structure containing the statistical models
%              for our healthy population generated using the PAPPI
%              framework. 
%
% Written by Brian Booth <brian.booth@uantwerpen.be>
%

  if ~exist('fit_linear_model'),
      p = mfilename('fullpath');
      [dir,~,~] = fileparts(p);
      addpath([dir, filesep, 'helper_functions']);
  end

  fprintf('Building Statistical Models...');
  
  % Convert cell array to a 3D matrix
  imgs = zeros([size(aligned_images{1}, 1), ...
                size(aligned_images{1}, 2), ...
                numel(aligned_images)]);
  for ii = 1:numel(aligned_images),
      imgs(:,:,ii) = aligned_images{ii};
  end
  
  % Create image mask
  mask_img = zeros(size(imgs(:,:,1)));
  for ii = 1:numel(aligned_images),
      mask_img = mask_img + threshold_scan(imgs(:,:,ii));
  end
  mask_img = double(mask_img > (0.05 * size(imgs,3)));
  
  % Fit PAPPI models
  [models.weights, models.residual_std, ~, ~] = ...
      fit_linear_model(imgs, demographics', mask_img);
  models.N = size(aligned_images,1);
  
  fprintf('Done!\n');