function [sig_image, raw_image] = compare_and_test(measured_image, ...
                                                   predicted_image, ...
                                                   models)
% function [sig_image, raw_image] = compare_and_test(measured_image, ...
%                                                    predicted_image, ...
%                                                    models)
%
% Compare an individual's peak pressure image to the peak pressure
% image predicted by the PAPPI statistical model. Highlight the
% peak pressures in the measured image that differ significantly
% from the prediction.
%
% Inputs:
%    - measured_image: The measured peak pressure image collected
%                      from an individual and aligned to the
%                      predicted image.
%    - predicted_image: The predicted peak pressure image obtained
%                       from the PAPPI model using the
%                       'predict_peak_pressures' function included
%                       in this software package.
%    - models: A MATLAB structure containing the PAPPI statistical
%              models that were calcuated using the 'build_models'
%              function in this software package.
%
% Outputs:
%    - sig_image: An image the same size as the predicted image
%                 that contains the statistically significant 
%                 one-sample t-statistics from the comparison
%                 between the measured and predicted
%                 images. Non-significant t-statistics are set to
%                 zero. 
%    - raw_image: An image the same size as the predicted image
%                 that contains all one-sample t-statistics from
%                 the comparison between the measured and predicted
%                 images, statistically significant or not.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>
%

  if ~exist('RFT'),
      p = mfilename('fullpath');
      [dir,~,~] = fileparts(p);
      addpath([dir, filesep, 'helper_functions']);
  end

  % Create mask image
  mask_img = threshold_scan(predicted_image);
  
  % Perform one sample t-tests at each pixel
  t_img = (measured_image - predicted_image) ./ ...
          max(eps, models.residual_std);
  t_img = t_img .* mask_img;
  raw_image = t_img * sqrt(models.N);
  
  % If the t-statistic is used and you have a large
  % degree-of-freedom, the z-score is roughly equivalent,
  % simpler, and more robust. Use it instead to correct for
  % multiple comparisons.
  if models.N > 30,
      raw_image = raw_image / sqrt(models.N);
      sig_mask = RFT(raw_image, mask_img, 0.05, 'Z', models.N);
  else
      sig_mask = RFT(raw_image, mask_img, 0.05, 'T', models.N);
  end 
  sig_image = raw_image .* sig_mask;
  
  % Trim very large t-statistics for display.
  idx = find(abs(sig_image) > 7);
  sig_image(idx) = sign(sig_image(idx)) * 7;
  