
% List here the paths to the CAD WALK datasets used in this
% example, both the healthy controls dataset and the hallux valgus
% dataset. Assuming that the datasets are properly downloaded into
% the provided data folder, the following code should find these
% datasets regardless of the computer this code runs on.

p = mfilename('fullpath');
[fdir,~,~] = fileparts(p);
addpath([fdir, filesep, '..']);

healthy_path = [fdir, filesep, '..', filesep, 'data', filesep, ...
                'HealthyControls']; 
halluxValgus_path = [fdir, filesep, '..', filesep, 'data', filesep, ...
                    'HalluxValgus_PreSugery']; 

%% Statistical Modelling Tasks

% Load the peak pressure measurements for the healthy controls.

[image_set, demographics] = load_cad_walk_data(healthy_path);

% Make the anatomically-unbiased peak pressure template.

template = make_template(image_set);

% Align all peak pressure images from the healthy controls to the
% anatomically-unbiased template

aligned_images = align_images(image_set, template);

% Build the statistical models that will serve as the baseline to
% which all patients can be compared.

models = build_models(aligned_images, demographics);


%% Statistical Testing Tasks 

% Load the peak pressure measurements for the hallux valgus
% patients

[image_set, demographics] = load_cad_walk_data(halluxValgus_path);

% For each patient, predict the baseline peak pressures for the
% patient, align their measured peak pressures to these
% predictions, and compare using single-sample t-tests. Create a
% cell array to save the results.

SPM_images = cell(size(image_set));

for ii = 1:numel(image_set),
    
    disp(sprintf('Testing patient %d', ii));
    
    % Predict the baseline peak pressures using the given
    % demographic factors
    baseline_image = predict_peak_pressures(demographics(:,ii), ...
                                            models);
    
    % Align the patient's measured peak pressures to their baseline
    aligned_image = align_images(image_set{ii}, baseline_image);
    
    % Compare measured pressure to the baseline using single-sample
    % t-tests.
    SPM_images{ii} = compare_and_test(aligned_image, ...
                                      baseline_image, models);
    
end
