function sig_mask = RFT(stat_img, mask_img, alpha, stat, df)
% function sig_mask = RFT(stat_img, mask_img, alpha, stat, df)
%
% Compute a multiple comparison corrected t-statistic threshold
% using Gaussian random field theory.
%
% Arguments:
%     - stat_img: The z-score, t-statistic or F-statistic image
%     - mask_img: A binary mask showing which t-statistics in the
%                 corresponding t-statistic image are valid.
%     - alpha: The level of significance (e.g. 0.05)
%     - stat: The choice of statistic in the image. Can be either
%             'T', 'Z', or 'F'.
%     - df: The number of degrees of freedom in the statistical
%           test. 
%
% Returns:
%     - sig_mask: A binary mask highlighting regions with alpha
%                 significance.  
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, July 28, 2016
%
      
    FWHM = calc_FWHM(stat_img, mask_img, stat, df);
    
    if ndims(mask_img) == 2,
        R = resels_2D(mask_img, FWHM);
    else
        R = resels_3D(mask_img, FWHM);
    end
    
    % Compute the expected Euler characteristic values for various
    % t-statistic thresholds 
    xx = 1:0.005:50;
    
    switch upper(stat),
      case 'T',
        rho = ec_t(xx, df);
      case 'Z',
        rho = ec_z(xx);
      case 'F',
        rho = ec_F(xx, df);
    end
    
    EC = sum(bsxfun(@times, rho, R(:)));
    
    % Select the threshold that is closest to alpha
    [~, idx] = min(abs(EC-alpha));
    t = xx(idx);
    
    % Bonferroni works better when image smoothness is lower. Compute the
    % Bonferroni threshold too and return the smaller of the two
    % options. 
    p_bon = 1 - (alpha / nnz(mask_img));
    switch upper(stat),
      case 'T',
        stat_bon = tinv(p_bon, df);
      case 'Z',
        stat_bon = norminv(p_bon);
      case 'F',
        stat_bon = finv(p_bon, df(1), df(2));
    end
    t = min(t, stat_bon);
    maskA = abs(stat_img) >= t;
    
    % Now, do the cluster inference
    sig_pval = RFT_Cluster(stat_img, mask_img, stat, df);
    maskB = sig_pval <= alpha;
    
    % subplot(1,3,1); imagesc(maskA); axis image; 
    % subplot(1,3,2); imagesc(maskB); axis image; 
    % subplot(1,3,3); imagesc(stat_img); axis image; colorbar;
    
    sig_mask = maskA | maskB;