function sig_pval = RFT_Cluster(stat_img, mask_img, stat, df)
% function sig_pval = RFT_Cluster(stat_img, mask_img, stat, df)
%
% Go through the given statistic image and identify clusters that
% as statistically significant based on their size.
%
% Arguments:
%     - stat_img: The z-score, t-statistic, or F-statistic image.
%     - mask_img: The binary mask for the region of interest.
%     - stat: The statistic represented in the statistic image. Can
%             be one of 'Z', 'T', or 'F'.
%     - df: The degrees of freedom associated with the statistical
%           test. 
%
% Returns:
%     - sig_pval: The p-vales image where the significant clusters
%                 have their p-value assigned to their pixels while
%                 the remaining pixels have a p-value of 1.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, July 29, 2016
%

    switch upper(stat),
      case 'Z',
        t = norminv(1 - 0.0025);
        rho = ec_z(t);
        seg = abs(stat_img) >= t;
      case 'T',
        t = tinv(1 - 0.001, df);
        rho = ec_t(t, df);
        seg = abs(stat_img) >= t;
      case 'F',
        t = finv(1 - 0.1, df(1), df(2));
        rho = ec_F(t, df);
        seg = stat_img >= t;
    end
    
    FWHM = calc_FWHM(stat_img, mask_img, stat, df);
    
    if ndims(mask_img) == 2,
        R = resels_2D(mask_img, FWHM);
    else
        R = resels_3D(mask_img, FWHM);
    end
    
    d = ndims(mask_img);
    
    EofN = R(d) * rho(1);
    EofM = R * rho;
    
    beta = (gamma(d/2 + 1) * (EofM / EofN)).^(2/d);
    
    label_img = my_bwlabeln(seg);
    sig_pval = ones(size(seg));
    
    % Go through each cluster and compute its probability
    for ii = 1:max(label_img(:)),
       
        c_sz = nnz(label_img == ii);
        p = 1 - exp(-EofM * exp(-beta * (c_sz .^ (2/d))));
        sig_pval(label_img == ii) = p;
        
    end
    