function FWHM = calc_FWHM(stat_img, mask_img, stat, df)
% function FWHM = calc_FWHM(stat_img, mask_img, stat, df)
%
% Compute the Full Width at Half-Max (FWHM) smoothness for the
% given statistical images. 
%
% Arguments:
%      - stat_image: The z, t, of F statistic image for which we
%                    will compute the FWHM.
%      - mask_img: The binary mask for the images so that we only
%                  compute the FWHM for the selected image region.
%      - stat: The type of statistic in the statistics image. Can
%              be one of 'Z', 'T', or 'F'.
%      - df: The effective number of degrees of freedom.
%
% Returns:
%      - FWHM: The full width at half-max smoothness value for the
%              given statistic image.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, July 28, 2016
%

    switch upper(stat),
      case 'T',
        stat_img = norminv(tcdf(stat_img, df));
        stat_img(isinf(stat_img)) = 0;
      case 'F',
        p_img = fcdf(stat_img, df(1), df(2));
        p_img = max(p_img, 1 - p_img) .* mask_img;
        stat_img = norminv(p_img);
        stat_img(isinf(stat_img)) = 0;
        df = sum(df);
    end
        
    FWHM = zeros(1, ndims(mask_img));
    N = nnz(mask_img);
    
    d = ndims(mask_img);
    if strncmp(upper(stat), 'Z', 1),
        c = 1;
    else
        % Compute correction term (Worsley, 2002)
        k = 1:(d+1);
        vk = (df - 1) - k + 1;
        pk = ones(size(k));
        pk(1) = -d;
        alpha = -1 / (2*d);
        c = (2 .^ (alpha * pk)) .* gamma(vk/2 + alpha*pk) ./ ...
            gamma(vk/2);
        c = prod(c);
    end
    
    if ndims(mask_img) == 2,
        [dx dy] = gradient(stat_img);
        M = [dx(:), dy(:)];
    else
        [dx dy dz] = gradient(stat_img);
        M = [dx(:), dy(:), dz(:)];
    end
    
    L = cov(M);
    FWHM = sqrt(4 * log(2) * (det(L) .^ (-1/d)));
    FWHM = FWHM / c;