function [cop, vop] = cop_vop(img, dt)
% function [cop, vop] = cop_vop(img, dt)
%
% Compute the center of pressure and velocity of the center of
% pressure values for the given plantar pressure image. Also,
% interpolate the resulting vaules to 101 time points in order to
% return the result as the percentage of the stance phase.
%
% Arguments:
%      - img: The plantar pressure image on which to do the
%             computation 
%      - dt: The time between plantar pressure measurements
%            (default is 5ms, equivaluent to 200 Hz)
%
% Returns:
%      - cop: A 101-by-2 matrix containing the (x,y) coordinates of
%             the center of pressure throughout the stance phase.
%      - vop: A 101-by-2 matrix containing the (dx,dy) velocities of
%             the center of pressure throughout the stance phase.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, July 15, 2016
%

    if nargin == 1 | numel(dt) == 0,
        dt = 5;
    end

    if numel(img) == 0,
        cop = zeros(1, 101);
        vop = zeros(1, 101);
        return;
    end
    
    mask_img = threshold_scan(img);
    cop_vec = zeros(size(img,3), 2);
    
    for ii = 1:size(img, 3),
       
        a_frame = img(:,:,ii);
        idx = find(img(:,:,ii));
        [xx yy] = ind2sub(size(a_frame), idx);
        img_vals = abs(a_frame(idx));
        cx = sum((img_vals .* xx) ./ max(eps, sum(img_vals(:))));
        cy = sum((img_vals .* yy) ./ max(eps, sum(img_vals(:))));
        if cx < 1 | cy < 1,
            cop_vec(ii,:) = [NaN NaN];
        else
            cop_vec(ii,:) = min([cx cy], [size(img,1) size(img,2)]);
        end
        
    end

    [r, c] = find(~isnan(cop_vec));
    r = unique(r);
    cop_vec = cop_vec(r,:);
    
    % Standardize the coordinate system
    [max_v, o_pt, min_v, foot_sz] = get_foot_system(img);
    
    % Step 1: Transfer all points to the new origin
    cop_vec2 = cop_vec - o_pt(ones(length(cop_vec),1), :);
        
    % Step 2: Vector projection for the x coordiante
    cop_final = zeros(size(cop_vec2,1), 2);
    cop_final(:,1) = real(sum(cop_vec2 .* repmat(max_v', length(cop_vec2), 1), 2));
    cop_final(:,1) = cop_final(:,1);
    
    % Step 3: Vector rejection for the y coordinate
    tmp_vecs = [cop_final(:,1) * max_v(1), cop_final(:,1) * max_v(2)];
    lat_vecs = cop_vec2 - tmp_vecs;
    cop_final(:,2) = real(sqrt(sum(lat_vecs .^ 2, 2)));
    
    % Step 4: Need sign for cop_final(:,2)
    cop_final(:,2) = real(cop_final(:,2) .* ...
        sign(sum(lat_vecs .* repmat(min_v', length(cop_vec2), 1), 2)));
    if cop_final(end,2) < 0,
        cop_final(:,2) = -cop_final(:,2);
        min_v = -min_v;
    end
        
    
    %% Test coordinate standardization
    % tmp_seg = max(mask_img, [], 3);
    % imagesc(tmp_seg); axis equal off;
    % hold on;
    % plot([o_pt(2); o_pt(2) + 300*max_v(2)], ...
    %      [o_pt(1); o_pt(1) + 300*max_v(1)], 'r');
    % plot(cop_vec(:,2), cop_vec(:,1), 'g');
    % plot([o_pt(2); o_pt(2) + 50*min_v(2)], ...
    %      [o_pt(1); o_pt(1) + 50*min_v(1)], 'r');
    % hold off;
    % drawnow;
    
    cop_vec = cop_final;
    
    % Now, compute velocities
    vop_vec = zeros(size(cop_vec, 1), 3);
    vop_vec(:,1) = gradient(cop_vec(:,1), dt);
    vop_vec(:,2) = gradient(cop_vec(:,2), dt);
    
    % Normalize for foot length and width
    cop_vec(:,1) = real(100 * cop_vec(:,1) ./ foot_sz(1));
    cop_vec(:,2) = real(100 * cop_vec(:,2) ./ foot_sz(2));
    
    % Interpolate to 101 samples
    old_x = linspace(0, 100, size(cop_vec,1));
    new_x = 0:100;
    cop = interp1(old_x, cop_vec, new_x, 'spline');
    vop = interp1(old_x, vop_vec, new_x, 'spline');
    cop = flip(cop, 2);
    vop = flip(vop, 2);
    
    % Finally, add the x and y velocities together.
    vop(:,3) = sqrt(vop(:,1).^2 + vop(:,2).^2);
    
    