function D = demons_register(fixed_image, moving_image)
% function D = demons_register(fixed_image, moving_image)
%
% Perform log-space diffeomorphic demons registration on the given
% 2D images.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, May 11, 2017
%

    % Setup algorithm options
    options.n_iter = 100;
    options.lambda_update = 2;
    options.lambda_deform = 2;
    options.lambda_image = 1;
    options.max_step = 1;
    options.n_levels = 2;

    % pad images to be of equal size
    end_sz = size(moving_image);
    [fixed_image, moving_image] = pad_to_equal_size(fixed_image, ...
                                                    moving_image);
    
    disp_field = zeros([size(moving_image) 2]);
    
    % For each level in the image pyramid, do registration
    for ii = options.n_levels:-1:1,
       
        % Downsample the images
        scale = 2.^(-(ii-1));
        fixed_low_res = imresize(fixed_image, scale);
        moving_low_res = imresize(moving_image, scale);
        disp_low_res = zeros([size(moving_low_res), 2]);
        disp_low_res(:,:,1) = imresize(disp_field(:,:,1) * scale, ...
                                       size(moving_low_res));
        disp_low_res(:,:,2) = imresize(disp_field(:,:,2) * scale, ...
                                       size(moving_low_res));
        
        % Perform the registration
        new_disp_field = do_registration(fixed_low_res, moving_low_res, ...
                                         disp_low_res, options);
                
        % Upsample resulting deformation field
        disp_field(:,:,1) = imresize(new_disp_field(:,:,1) / scale, ...
                                     size(moving_image));
        disp_field(:,:,2) = imresize(new_disp_field(:,:,2) / scale, ...
                                     size(moving_image));
        
    end
    
    % exponentiate the log-space deformation
    D = exp_disps(disp_field);
    D = D(1:end_sz(1), 1:end_sz(2), :);
    
    % tmp_img = warp_image(moving_image, D);
    % imshowpair(tmp_img, fixed_image);
    % drawnow;
    
% main, non-hierarchical log-demons registration function
function disps_best = do_registration(fixed_image, moving_image, ...
                                      init_disps, options)

    % add some extra room for boundary conditions
    ex_pad = round(0.1 * size(moving_image));
    fixed_image = padarray(fixed_image, ex_pad);
    moving_image = padarray(moving_image, ex_pad);
    init_disps = padarray(init_disps, [ex_pad 0]);

    % setup energy minimization stopping criteria data
    eng = zeros(1, options.n_iter);
    eng_min = 1e20;
    eng_t = 0.001;
    iter_step = 5;
    disp_field = init_disps;
    disps_best = disp_field;

    % perform all iterations
    for ii = 1:options.n_iter,
    
        % Compute the gradient step
        disp_update = get_step(fixed_image, moving_image, disp_field, ...
                                            options.lambda_image, ...
                                            options.max_step);
        
        % Regularize displacement updates
        disp_update = smooth_field(disp_update, ...
                                   options.lambda_update);
        
        % Update displacements
        disp_field = disp_field + options.max_step * disp_update;
                
        % Regularize displacement field
        disp_field = smooth_field(disp_field, ...
                                  options.lambda_deform);
        
        % Get energy function value for stopping criteria
        eng(ii) = calc_energy(fixed_image, moving_image, ...
                              disp_field, options.lambda_image, ...
                              options.max_step);
        
        % Check if the step minimized the energy
        if eng(ii) < eng_min,
            disps_best = disp_field;
            eng_min = eng(ii);
        end
        
        %fprintf('Iteration %d - energy %g\n', ii, eng(ii));
        
        % Test stopping critera
        if ii > 1 & (abs(eng(ii) - eng(max(1, ii-iter_step))) < ...
                     eng(1)*eng_t);
            break;
        end
        
    end
    
    disps_best = disps_best(ex_pad(1)+1:end-ex_pad(1), ...
                            ex_pad(2)+1:end-ex_pad(2),:);
    
% Perform a gradient step in the log-demons algorithm
function update = get_step(fixed_image, moving_image, disp_field, ...
                                        lambda, max_dt);
    
    % Get the moving image under the current deformation
    D = exp_disps(disp_field);
    deform_image = warp_image(moving_image, D);
        
    % Calculate the image difference
    image_diff = fixed_image - deform_image;
    
    % Get the gradient of the difference image
    [dy, dx] = gradient(deform_image);
    grad_norm = dx.^2 + dy.^2;
    
    % Scale gradient update
    scale = image_diff ./ (grad_norm + (image_diff.^2) * ...
                           (lambda / max_dt).^2);
    scale(grad_norm == 0) = 0;
    scale(image_diff == 0) = 0;
    
    % Make gradient update
    update = zeros(size(disp_field));
    update(:,:,1) = dx .* scale;
    update(:,:,2) = dy .* scale;
            
% calculate the value of the energy function
function eng = calc_energy(fixed_image, moving_image, disp_field, ...
                           lambda, max_dt);
   
    % Get the moving image under the current deformation
    D = exp_disps(disp_field);
    deform_image = warp_image(moving_image, D);
    
    % imshowpair(fixed_image, deform_image);
    % drawnow;
    % pause(1);
    
    % Compute the image difference
    image_diff = (fixed_image - deform_image).^2;
    
    % Calculate the jacobian determinant of the deformation
    [dy_x, dx_x] = gradient(disp_field(:,:,1));
    [dy_y, dx_y] = gradient(disp_field(:,:,2));

    dx_x = dx_x + 1; % Add identity so that zero displacement ...
    dy_y = dy_y + 1; % gives an identity transformation.
    
    jac_deform = (dx_x .* dy_y) - (dy_x .* dx_y);
        
    % Normalize the energy components
    eng_image = sum(image_diff(:)) ./ numel(image_diff);
    eng_deform = sum(jac_deform(:).^2) ./ numel(jac_deform);
    
    % Combine to calculate energy
    eng = eng_image + ((lambda / max_dt).^2) * eng_deform;
    
% exponentiate the log-space deformation
function D = exp_disps(disp_field)

    % Find a factor N that scales the displacements to < 0.5mm
    vec_norms = sum(disp_field.^2, 3);
    max_dist = real(sqrt(max(vec_norms(:))));
    N = ceil(log2(max_dist/0.5));
    N = max(N, 0);
    
    % Scale displacements down by factor of 2^N.
    % This small scale displacement vector field is guaranteed to
    % be a diffeomorphism
    disp_field = disp_field * (2.^(-N));
    D = disp_field;
    
    % Square the displacement field N times to get diffeomorphism
    for ii = 1:N,
        D = concatenate_disps(D, D);
    end
    
% Concatenate diffeomorphisms
function D = concatenate_disps(disps1, disps2);

    % interpolate the displacements for disps2 to the displaced
    % pixel coordinates from disps1
    interp_disps2 = zeros(size(disps2));
    interp_disps2(:,:,1) = warp_image(disps2(:,:,1), disps1);
    interp_disps2(:,:,2) = warp_image(disps2(:,:,2), disps1);
    
    % combine displacements
    D = disps1;
    D(:,:,1) = D(:,:,1) + interp_disps2(:,:,1);
    D(:,:,2) = D(:,:,2) + interp_disps2(:,:,2);
    
% Warp the given image with the given deformation
function new_image = warp_image(old_image, D)

    % get pixel coordinates
    [px, py] = meshgrid(0:size(old_image,2)-1, 0:size(old_image,1)-1);
    new_x = px + D(:,:,2);
    new_y = py + D(:,:,1);
    
    % interpolate new image at given positions
    new_image = interp2(px, py, old_image, new_x, new_y, 'linear', 0);
    
% Smooth the given deformation vector field with a gaussian filter
function new_field = smooth_field(disp_field, sigma)

    % Check for no smmothing case
    if sigma == 0,
        new_field = disp_field;
        return;
    end

    % Create a gaussian kernel
    r = ceil(3*sigma);
    [fx, fy] = ndgrid(-r:r, -r:r);
    h = exp(-(fx.^2 + fy.^2) / (2*sigma*sigma));
    h = h ./ sum(h(:));
    
    % filter the given displacements
    new_field = disp_field;
    new_field(:,:,1) = conv2(new_field(:,:,1), h, 'same');
    new_field(:,:,2) = conv2(new_field(:,:,2), h, 'same');
    
% pad fixed and moving images to be of equal size.
function [fixed_image, moving_image] = pad_to_equal_size(fixed_image, ...
                                                  moving_image);

    % Get padding sizes
    imSz_F = size(fixed_image);
    imSz_M = size(moving_image);
    new_imSz = max(imSz_F, imSz_M);
    
    pad_sz_F = (new_imSz - imSz_F);
    pad_sz_M = (new_imSz - imSz_M);
    
    % pad images
    fixed_image = padarray(fixed_image, pad_sz_F, 'post');
    moving_image = padarray(moving_image, pad_sz_M, 'post');
        