function map_idx = dynamic_time_warping(fixed_img, moving_img)
% function map_idx = dynamic_time_warping(fixed_img, moving_img)
%
% Compute the dynamic time warping between the given 2D+time images
% so as to provide a discrete mapping that aligns the image along
% the temporal dimension. This function assumes that the given
% images have already been spatially aligned. To align the images
% along the temporal axis, we use XOR of the thresholded
% segmentations from the two given images.
%
% Arguments:
%       - fixed_img: The fixed image that we will register to.
%       - moving_img: The moving image that will be registered.
%
% Returns:
%       - map_idx: The frames in the fixed image that correspond to
%                  the frames in the moving image.
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 6, 2016
%


    fixed_img = threshold_scan(fixed_img(1:min(size(fixed_img,1), ...
                                               size(moving_img,1)), ...
                                         1:min(size(fixed_img,2), ...
                                               size(moving_img,2)),:));
    moving_img = threshold_scan(moving_img(1:min(size(fixed_img,1), ...
                                                 size(moving_img,1)), ...
                                           1:min(size(fixed_img,2), ...
                                                 size(moving_img,2)),:));
    
    frame_dists = zeros(size(fixed_img,3)+1, size(moving_img,3)+1);
    frame_prev = zeros(size(fixed_img,3)+1, size(moving_img,3)+1);
    
    frame_dists(1,2:end) = inf;
    frame_dists(2:end,1) = inf;
    
    for ii = 2:size(fixed_img,3)+1,
        for jj = 2:size(moving_img,3)+1,
            pair_cost = xor_dist(fixed_img(:,:,ii-1), ...
                                 moving_img(:,:,jj-1));
                        
            if frame_dists(ii-1, jj) < frame_dists(ii, jj-1) & ...
               frame_dists(ii-1, jj) < frame_dists(ii-1, jj-1),
                frame_dists(ii,jj) = frame_dists(ii-1, jj) + ...
                    pair_cost;
                frame_prev(ii,jj) = sub2ind(size(frame_prev), ...
                                            ii-1, jj);
            elseif frame_dists(ii, jj-1) < frame_dists(ii-1, jj) & ...
                   frame_dists(ii, jj-1) < frame_dists(ii-1, jj-1),
                frame_dists(ii,jj) = frame_dists(ii, jj-1) + ...
                    pair_cost;
                frame_prev(ii,jj) = sub2ind(size(frame_prev), ...
                                            ii, jj-1);
            else
                frame_dists(ii,jj) = frame_dists(ii-1, jj-1) + ...
                    pair_cost;
                frame_prev(ii,jj) = sub2ind(size(frame_prev), ...
                                            ii-1, jj-1);
            end
        end
    end
    
    %frame_dists = frame_dists(2:end, 2:end);
    
    path = zeros(1, 2*size(fixed_img,3));
    k = 1;
    path(k) = numel(frame_prev);
    while true,
        if frame_prev(path(k)) == 0,
            break
        end
        k = k + 1;
        path(k) = frame_prev(path(k-1));
    end
    path = path(1:k);
    
    [fixed_idx, moving_idx] = ind2sub(size(frame_prev), path);
    
    moving_idx = moving_idx - 1;
    moving_idx(moving_idx == 0) = [];
    fixed_idx = fixed_idx - 1;
    fixed_idx(fixed_idx == 0) = [];
    
    if moving_idx(end) ~= 1,
        % imagesc(frame_prev); axis equal off; colorbar;
        % figure;
        % imagesc(frame_dists); axis equal off; colorbar;
        % drawnow;
        % pause;
        to_add = moving_idx(end)-1:1;
        moving_idx = [moving_idx to_add];
        fixed_idx = [fixed_idx ones(size(to_add))];
    end
    if fixed_idx(end) ~= 1,
        to_add = fixed_idx(end)-1:1;
        fixed_idx = [fixed_idx to_add];
        moving_idx = [moving_idx ones(size(to_add))];
    end
    
    k = numel(fixed_idx) - numel(moving_idx);
    if k > 0
        tmp_idx = padarray(moving_idx, [0 k], 1, 'post');
        moving_idx = tmp_idx(1,:);
    elseif k < 0
        tmp_idx = padarray(fixed_idx, [0 k], 1, 'post');
        fixed_idx = tmp_idx(1,:);
    end
    
    map_idx = [flip(fixed_idx(:)) flip(moving_idx(:))];
    
    
% Compute the exclusive or between the thresholded versions of the
% two given images. Assuming both images are the same size (which
% should be the case if they are already spatially aligned).
function d = xor_dist(img1, img2)

    %d = nnz(img1 & img2) ./ nnz(img1 | img2);
    d = sqrt(sum(sum((img1 - img2).^2)));