function [w_img, err_img, p_img, residuals] = fit_linear_model(imgs, covars, ...
                                                    mask_img, lambda)
% function [w_img, err_img, p_img] = fit_linear_model(imgs, covars, ...
%                                                     mask_img, lambda)
%
% Fit a linear model at each pixel in the image using the given
% matrix of covariates. Optionally generate a sparse linear model
% with the given sparsity weight.
%
% Arguments:
%      - imgs: A 4D array of aligned plantar pressure images.
%      - covars: An N-by-d matrix where N is the number of images
%                and d is the number of covariates. All covariates
%                are assumed to be numeric.
%      - mask_img: A binary mask of the footprint.
%      - lambda: zero if no sparsity constraint is to be
%                used (default). Otherwise, this variable contains
%                the weight of the sparsity penalty in the lasso
%                regression. 
%
% Returns:
%      - w_img: A 4D image with the fourth dimension holding the
%               linear regression weights for each covariate. The
%               first entry in that dimension is the bias value.
%      - err_img: A 4D image containing the standard error of each
%                 fitted linear regression model.
%      - p_img: A 4D image containing the p-values and r-squared
%               values of the model fit.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, April 26, 2016
%

  if nargin == 3,
      lambda = 0;
  end
  
  if ndims(imgs) == 3,
      imgs = reshape(imgs, ...
                     [size(imgs,1) size(imgs,2) 1 size(imgs,3)]);
      mask_img = max(mask_img, [], 3);
  end
  
  imSz = size(imgs(:,:,:,1));
  if numel(imSz) == 2,
      imSz = [imSz 1];
  end
  
  covars = [ones(size(covars,1),1), covars];
  
  % Remove covariates for images we don't have.
  [idd, c] = find(isnan(covars));
  covars(idd,:) = [];
  
  % Remove covariates that are constant
  std_covars = std(covars(:,2:end));
  idd = find(std_covars < 1e-5) + 1;
  covars(:,idd) = [];
  
  idx = find(mask_img);
  
  imgs = reshape(imgs, [prod(imSz) size(imgs,4)]);
  
  w_img = zeros(size(imgs,1), size(covars,2));  
  warning('off','all');
  if lambda == 0,
      for ii = 1:numel(idx),
          try
              w_img(idx(ii),:) = (covars \ (imgs(idx(ii), :)'))';
          catch ME
              whos
              size(w_img(idx(ii),:))
              size(imgs(idx(ii), :)')
              ii
              idx(ii)
              rethrow(ME);
          end
      end
  else
      for ii = 1:numel(idx),
          w_img(idx(ii),:) = lasso(covars, imgs(idx(ii),:), ...
                                   'Lambda', lambda)';
      end
  end
  warning('on', 'all');

  w_img = reshape(w_img, [imSz size(covars,2)]);

  % Compute the predicted plantar pressure vaules from the given
  % linear model. Needed for the standard error calculation.
  pred_imgs = w_img(:,:,:,ones(size(imgs, 2), 1));
  for ii = 2:size(covars,2),
     
      pred_imgs = pred_imgs + ...
          w_img(:,:,:,ii*ones(size(imgs, 2), 1)) .* ...
          repmat(reshape(covars(:,ii), [1 1 1 size(covars,1)]), ...
                 [imSz 1]);
      
  end
  
  % Add in zero weight images for covariates that were removed from
  % the modelling
  new_img = zeros([imSz, size(covars,2) + numel(idd)]);
  new_img(:,:,:,1:size(w_img,4)) = w_img;
  for ii = 1:numel(idd),
      new_img(:,:,:,idd(ii)+1:end) = ...
          new_img(:,:,:,idd(ii):end-1);
      new_img(:,:,:,idd(ii)) = 0;
  end
  w_img = new_img;
    
  % Compute the standard error
  imgs = reshape(imgs, [imSz, size(imgs,2)]);
  residuals = imgs - pred_imgs;
  err_img = sum((imgs - pred_imgs).^2, 4) ./ ...
            (size(covars,1) - size(covars,2));
  
  % Compute the regression error
  mean_img = mean(imgs, 4);
  reg_img = sum((pred_imgs - ...
                 mean_img(:,:,:,ones(1, size(pred_imgs,4)))) ...
                .^2, 4) ./ (size(covars,2) - 1);
  
  % Compute the p-values for each individual covariate weight
  err_vec = diag(inv(covars' * covars + 1e-8*eye(size(covars,2))));
  t_img = zeros(size(w_img));
  for ii = 1:numel(err_vec),
      t_img(:,:,:,ii) = sqrt(err_img .* err_vec(ii));
  end
  p_img = 2 * (1 - tcdf(abs(w_img ./ max(eps, t_img)), ...
               size(covars,1) - size(covars,2)));
  p_img = min(p_img, 1-p_img);
    
  % Compute the p-value for the the linear model as a whole
  F_img = reg_img ./ max(eps, err_img);
  p2_img = fcdf(F_img, size(covars,2)-1, ...
                size(covars,1) - size(covars,2));
  p2_img = min(p2_img, 1-p2_img);
  
  % Compute the r-sqared of the model fit.
  var_img = max(eps, var(imgs, 0, 4));
  r2_img = 1 - err_img ./ var_img;

  % Compute the normalcy values for the residuals
  %tmp_img = imgs - pred_imgs;
  %p_img(:,:,:,end+1) = lillietest_vec(tmp_img, mean(tmp_img, 4), ...
  %                                    var(tmp_img, 0, 4));
  
  % Combine all godness-of fit & significance tests into one set of
  % images
  p_img(:,:,:,end+1) = p2_img;
  p_img(:,:,:,end+1) = r2_img;
  
  % Get standard error from mean-squared error
  err_img = sqrt(err_img);