function [max_v, o_pt, min_v, foot_sz] = get_foot_system(img)
% function [max_v, o_pt, min_v, foot_sz] = get_foot_system(img)
%
% Compute the x and y axis of the fot provided in the given plantar
% pressure image. The x-axis is the anterior-posterior axis and
% runs from the back of the heel to the top of the second toe. The
% y-axis is perpendicular to that.
%
% Arguments:
%      img - The 2D+time plantar pressure image.
%
% Returns:
%      max_v - A vector pointing along the anterior-posterior
%              direction. 
%       o_pt - The origin point on the back of the heel.
%      min_v - A vector pointing along the medial-lateral
%              direction. 
%    foot_sz - The length and width, in pixels, of the foot.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, July 18, 2016
%

    mask_img = threshold_scan(img);
    mask_img = max(mask_img, [], 3);
    [xx yy] = find(mask_img);
    
    % Do PCA with the segmentation points to get the main axes.
    pt = [mean(xx) mean(yy)];
    aa = xx - mean(xx);
    bb = yy - mean(yy);
    A = [aa(:) bb(:)];
    [vecs, vals] = eig(A' * A);
    
    if abs(vecs(1,2)) > abs(vecs(1,1)),
        vecs = flip(vecs,2);
    end
    
    % Extract the anterior-posterior axis.
    max_v = vecs(:,1);
    min_v = -vecs(:,2);
    
    % Use the mean point above to get the equation of the line
    m = max_v(2) ./ max_v(1);
    b = pt(2) - m * pt(1);
    
    % Get points at the top and bottom of the foot to plot the
    % line. 
    y_top = m * 1 + b;
    y_btm = m * size(mask_img,1) + b;
    
    theta = acos(dot(vecs(:,1), [-1, 0]));
    R = [cos(theta) -sin(theta); sin(theta) cos(theta)];
    
    %imagesc(mask_img); axis equal off;
    %c_pts = bwmorph(mask_img, 'remove');
    c_pts = my_bwperim(mask_img);
    % imshow(c_pts);
    % drawnow;
    % pause(1);
    c1 = zeros(nnz(c_pts), 2);
    [c1(:,2), c1(:,1)] = find(c_pts);
    c1 = c1';
    %c1 = contourc(double(mask_img), 1); %axis equal;
                                        %hold on;
    uu = [1, size(mask_img,1)];
    vv = [y_top, y_btm];    
    
    [xi, yi] = my_curve_intersect(uu, vv, c1(2,:), c1(1,:));
    %[xi, yi] = polyxpoly(uu, vv, c1(2,:), c1(1,:));
    [~, idx] = max(xi);
    
    if numel(idx) == 0,
        o_pt = [uu(2) vv(2)];
        % contour(double(mask_img), 1); axis equal;
        % hold on;
        % plot(vv(:), uu(:), 'r');
    else
        o_pt = [xi(idx) yi(idx)];
    end
    
    % o_pt = [xi(idx) yi(idx)];

    % contour(double(mask_img), 1); axis equal;
    % hold on;
    % plot(vv(:), uu(:), 'r');
    % plot(o_pt(2), o_pt(1), 'bo');
    % hold off;
    % drawnow;    
    % pause;
    
    % Now compute foot length and width
    pts = [xx(:) yy(:)];
    
    pts = pts - o_pt(ones(length(pts), 1), :);
    
    all_lengths = sum(pts .* repmat(max_v', length(pts), 1), 2);
    
    if (nnz(all_lengths < 0) / numel(all_lengths)) > 0.05 | ...
        size(mask_img,1) - o_pt(1) > size(mask_img,1)/2,
        % Origin is wrong. Heel probably cut off
        o_pt = [size(mask_img,1) real(y_btm)];
        pts = [xx(:) yy(:)];
        pts = pts - o_pt(ones(length(pts), 1), :);
        
        all_lengths = sum(pts .* repmat(max_v', length(pts), 1), 2);
    end
    
    foot_length = max(sum(pts .* repmat(max_v', length(pts), 1), 2));
    
    all_widths = sum(pts .* repmat(min_v', length(pts), 1), 2);
    foot_width = max(all_widths) - min(all_widths);
    
    %plot(all_widths(:), all_lengths(:), '+');
    %axis equal;
    %drawnow;
    
    foot_sz = [foot_length foot_width];

    arch_side = mean(all_lengths(all_widths < 0 & ...
                                 all_lengths > ...
                                 (max(all_lengths)/2)));
    toe_side = mean(all_lengths(all_widths >= 0 & ...
                                all_lengths > ...
                                (max(all_lengths)/2)));
    
    if arch_side > toe_side,
        min_v = -min_v;
    end
    
    %pause;
    
