function [left_avg, right_avg, left_var, right_var] = ...
        intrasubject_avg_img(img_set, wb)
% function [left_avg, right_avg, left_var, right_var] = ... 
%       intrasubject_avg_img(img_set, wb)
%
% Take an average of a single subject's plantar pressure
% images. Generate a separate average for each foot. Use affine
% registration to align the plantar pressure images.
%
% Arguments:
%      - img_set: A set of sanitized plantar pressure images from
%                 the same subject. This set is created when
%                 running the load_ppressure_scan.m script.
%      - wb: An optional handle to a progress bar.
%
% Returns:
%      - left_avg: The average peak pressure scan for the left foot
%                  of the given subject.
%      - right_avg: The average peak pressure scan for the left foot
%                   of the given subject.
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 6, 2016
%

    % Identify the set of images for the left and right feet.
    left_img = zeros(1,0);
    right_img = zeros(1,0);
    
    if isempty(img_set),
        left_var = zeros(1,0);
        left_avg = zeros(1,0);
        right_avg = zeros(1,0);
        right_var = zeros(1,0);
        return;
    end
    
    for ii = 1:numel(img_set.imgs),
        if strncmp(img_set.imgs(ii).foot_side, 'left', 4)
            left_img = [left_img ii];
        else
            right_img = [right_img ii];
        end
    end
    
    if nargin == 2,
        p.n_steps = numel(img_set.imgs) + 1;
        p.curr_step = 0.2;
        p.wb = wb;
        
        % For the Left Foot
        [left_avg, left_var] = calc_mean_img(img_set, left_img, p);
        p.curr_step = p.curr_step + 0.8*(numel(left_img) / p.n_steps);
        % For the Right Foot
        [right_avg, right_var] = calc_mean_img(img_set, right_img, p);
    else
        % For the Left Foot
        [left_avg, left_var] = calc_mean_img(img_set, left_img);
        % For the Right Foot
        [right_avg, right_var] = calc_mean_img(img_set, right_img);
    end
    
% Computes the average of the planter pressure maps indexed by 'idx'
function [avg_img, var_img] = calc_mean_img(img_set, idx, p)

    if numel(idx) == 0,
        avg_img = [];
        var_img = [];
        return;
    end
    
    % images = cell(numel(idx),1);
    
    % for ii = 1:numel(idx),
    %     images{ii} = img_set.imgs(idx(ii)).img;
    % end
    
    % avg_img = hierarchical_average(images);
    % var_img = zeros(size(avg_img));
    
    % imagesc(max(avg_img, [], 3)); axis image; colormap hot;
    % drawnow; pause(0.5);
    
    % for ii = 1:numel(idx),
    %     pad_sz = max(0, size(avg_img) - size(images{ii}));
    %     tmp_img = padarray(images{ii}, pad_sz, 'post');
    %     tmp_img = tmp_img(1:size(avg_img,1), 1:size(avg_img,2), ...
    %                       1:size(avg_img,3));
    %     var_img = var_img + (tmp_img - avg_img) .^ 2;
    % end
    % var_img = var_img ./ numel(idx);
    %idx = idx(1:8);
    
    % Step 1: Take the first image as the template
    template = padarray(img_set.imgs(idx(1)).img, [6 6]);
    aligned_imgs = zeros(size(template));
    mask_imgs = zeros(size(template));
        
    % Step 2: For all other scans of that foot, register their peak
    % pressure maps to the template.
    for ii = 1:numel(idx),
        %disp(idx(ii));
        [tmp_img, ~, tmp_mask] = spatiotemporal_register(...
            template, img_set.imgs(idx(ii)).img, 'rigid');
        aligned_imgs = aligned_imgs + tmp_img;
        
        % imshowpair(max(tmp_img, [] ,3), max(template, [], 3));
        % drawnow;
        % pause;
        
        mask_imgs = mask_imgs + tmp_mask;
    
        if nargin == 3,
            waitbar(p.curr_step + 0.8*(ii / p.n_steps), p.wb, 'Aligning Trials...');
        end
    end
    
    % Step 3: Take the mean of the aligned images.
    avg_img = aligned_imgs ./ max(1, mask_imgs);
    avg_img = avg_img .* (mask_imgs > (numel(idx) / 2));
    
    % Step 4: Crop average image to bounding box size + 1
    [rr cc] = find(max(avg_img, [], 3));
    avg_img = avg_img(min(rr):max(rr), min(cc):max(cc), :);
    avg_img = padarray(avg_img, [1 1 0], 'both');
    
    % Step 5: Get the within-subject variance
    var_img = var(aligned_imgs, 0, 4);
    
    % imagesc(max(avg_img, [], 3)); axis image; colormap hot;
    % drawnow; pause;
