function T = linear_register2(fixed_img, moving_img, tform_type)
% function T = linear_register(fixed_img, moving_img, tform_type)
%
% Perfom a 2D linear image registration between the given
% images. The registration can be based on either a rigid or an
% affine transformation.
%
% Arguments:
%       - fixed_img: The 2D image we want to align to.
%       - moving_img: The 2D image we want to transform.
%       - tform_type: The type of image transformation. Options are
%                     'rigid' or 'affine'.
%
% Returns:
%       - T: The 3x3 transformation matrix that linearly aligns the
%            two images.
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, May 23, 2017
%
    
    [fixed_img, moving_img] = pad_to_equal_size(fixed_img, ...
                                                moving_img);
    
    if strncmp(tform_type, 'rigid', 5)
        params = [0, 0, 0];
        % old_fixed = fixed_img;
        % old_moving = moving_img;
        % fixed_img = imresize(fixed_img, 0.5);
        % moving_img = imresize(moving_img, 0.5);
    else
        params = [0, 0, 0, 1, 1];
        % old_fixed = fixed_img;
        % old_moving = moving_img;
        % fixed_img = imresize(fixed_img, 0.5);
        % moving_img = imresize(moving_img, 0.5);
    end
    
    params = evolve_optimize(fixed_img, moving_img, params);
    
    % if strncmp(tform_type, 'rigid', 5)
    %     params(1:2) = (1/0.5)*params(1:2);
    % else
    %     params(1:2) = (1/0.5)*params(1:2);
    % end
    
    T = make_transformation(moving_img, params);
    
    % Check the result
    % new_img = transform_image(old_moving, params);
    % imshowpair(old_fixed, new_img);
    % drawnow;
    % pause(1);    
    
    
% optimize using the 1+1 evolutionary solver
function tform_final = evolve_optimize(fixed_img, moving_img, tform_init)

    if numel(tform_init) == 3,
        sigma = [0.5; 0.5; 0.5];
        max_iter = 200;
    else
        sigma = [0.5; 0.5; 0.5; 7e-3; 7e-3];
        max_iter = 250;
    end
    curr_img = moving_img;
    obj = 2;  % Maximum value for normalized mututal information
        
    n       = length(tform_init);     % 'n' states
    xk      = zeros(n, max_iter);     % Pre-allocate space for 'xk'
    maxE    = 1.5e-3;                 % maximum error
    k       = 1;                      % counter
    e       = 10;                     % initial high error
    xk(:,1) = tform_init;             % initial guessing
    fx      = mutual_information(fixed_img, curr_img);  % evaluate
                                                        % function
                                                        % with
                                                        % initial
                                                        % transform
    A = diag(sigma);
    cc = 1.025;
    
    while ((k < max_iter) && (e > maxE))
        r = randn(n,1);
        y  = xk(:,k) + A*r; % create mutated solution 'y'
        curr_img = transform_image(moving_img, y);
        fy = mutual_information(fixed_img, curr_img); % evaluate MI
                                                      % with
                                                      % mutated
                                                      % solution
                                                      % 'y' 
        if (fy > fx)
            xk(:,k+1) = y;                  % update value of transform
            fx        = fy;                 % update value of MI
            A = A*(eye(size(A)) + ...       % update and grow covariance
                   ((cc-1)/(r'*r) * (r*r')));
        else
            xk(:,k+1) = xk(:,k);            % retain value of transform
            A = A*(eye(size(A)) + ...       % update and shrink covariance
                   (((1/cc)-1)/(r'*r) * (r*r')));
        end
        e = abs(obj - fx);              % error
        k = k+1;                        % update counter
    end

    tform_final = xk(:,k);

    
% compute the mutual information of the given image pair
function MI = mutual_information(img1, img2)
    
    binIDA = min(50, max(1, round(50*(img1 ./ max(img1(:))) + 0.5)));
    binIDB = min(50, max(1, round(50*(img2 ./ max(img2(:))) + 0.5)));
    pAB = zeros(50);
    for ii = 1:50,
        for jj = 1:50,
            pAB(ii,jj) = nnz((binIDA == ii) & (binIDB == jj));
        end
    end
    pAB = pAB ./ sum(pAB(:));
    
    entA = -sum(sum(pAB,2) .* log(max(eps, sum(pAB, 2))));
    entB = -sum(sum(pAB,1) .* log(max(eps, sum(pAB, 1))));
    entAB = -sum(pAB(:) .* log(max(eps, pAB(:))));
    MI = (entA + entB) / (entAB + ~(entAB|0));
        
    
% create and apply a linear transformation to the given image
function new_img = transform_image(img, params)
    
    T = make_transformation(img, params);
    new_img = my_img_warp(img, T, [], size(img));

% compute the transformation matrix from the parameters
function T = make_transformation(img, params)
    
    DX = [1 0 0; 0 1 0; params(1) params(2) 1];
    R = [cosd(params(3)) -sind(params(3)) 0; ...
         sind(params(3)) cosd(params(3)) 0; ...
         0 0 1];
    c_pt = size(img) / 2;
    R = [1 0 0; 0 1 0; -c_pt(1) -c_pt(2) 1] * R * ...
        [1 0 0; 0 1 0; c_pt(1) c_pt(2) 1];
    if numel(params) > 3,
        S = diag([params(4) params(5) 1]);
    else
        S = eye(3);
    end
    
    T = S * DX * R;
    
% pad fixed and moving images to be of equal size.
function [fixed_img, moving_img] = pad_to_equal_size(fixed_img, ...
                                                     moving_img);

    % Get padding sizes
    imSz_F = size(fixed_img);
    imSz_M = size(moving_img);
    new_imSz = max(imSz_F, imSz_M);
    
    pad_sz_F = (new_imSz - imSz_F);
    pad_sz_M = (new_imSz - imSz_M);
    
    % pad images
    fixed_img = padarray(fixed_img, pad_sz_F, 'post');
    moving_img = padarray(moving_img, pad_sz_M, 'post');
    