function [left_avg, right_avg] = make_unbiased_mean_sb(subject_imgs, img_type)
% function [left_avg, right_avg] = make_unbiased_mean_sb(subject_imgs, img_type)
%
% Create an unbiased mean image from a list of plantar pressure
% scans provided in the given file. The user can choose between a
% peek pressure mean image or a full 2D+time image. This version
% uses the atlas creation technique of Guimond et al., CVIU 2000.
%
% Arguments:
%      - subject_imgs: The plantar pressure images to create the
%                      unbiased mean image from.
%      - img_type: The type of mean image to create. The options
%                  are 'peak' for a peek pressure mean image, and
%                  'full' for the full 2D+time mean image.
%
% Returns:
%      - left_avg: The unbiased mean image for the left foot.
%      - right_avg: The unbiased mean image for the right foot.
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 11, 2016
%
    
    fprintf(1, 'Creating average image:\n');
    
    % Now that we have each subject's foot images, begin computing
    % the unbiased mean image. Add some size to the template images
    % to make sure that the feet end up in the image frame.
    template{1} = padarray(subject_imgs{1,1}, [8 8]);

    if size(subject_imgs,2) > 1,
        template{2} = padarray(subject_imgs{1,2}, [8 8]);
    else
        template{2} = [];
    end
        
    for num_iter = 1:2,
        fprintf(1, '\tIteration %d of 2...', num_iter);
        template = guimond_atlas(subject_imgs, template, ...
                                     img_type);
        fprintf(1, 'Done!\n');
    end
    
    left_avg = template{1};
    
    if size(subject_imgs,2) > 1,
        right_avg = template{2};    
    else
        right_avg = [];
    end
    
    
% Create an unbiased mean image from the given set of images. Use
% the given template as the original biased reference space.
function atlas = guimond_atlas(subject_imgs, template, img_type)
    
    % Register each scan to the given template. Do so for both the left
    % and right foot.
    aligned_imgs = subject_imgs;
    transforms = cell(size(aligned_imgs));
    for kk = 1:size(subject_imgs,1),
        for aa = 1:size(subject_imgs,2),
            if numel(subject_imgs{kk,aa}) == 0,
                transforms{kk,aa}.affineMat = zeros(3);
                transforms{kk,aa}.disp_field = [];
                transforms{kk,aa}.time_map = [];
                continue,
            end
            
            if strncmp(img_type, 'peak', 4),
                [aligned_imgs{kk,aa}, affine_mat, disp_field] = ...
                    scalar_img_register(template{aa}, subject_imgs{kk,aa}, ...
                                      'deformable');
                transforms{kk,aa}.affineMat = affine_mat;
                transforms{kk,aa}.disp_field = disp_field;
                transforms{kk,aa}.time_map = [];
            else
                [aligned_imgs{kk,aa}, transforms{kk,aa}] = ...
                    spatiotemporal_register(template{aa}, subject_imgs{kk,aa}, ...
                                            'deformable');
                % imshowpair(max(aligned_imgs{kk,aa}, [], 3), ...
                %            max(template{aa}, [], 3));
                % drawnow; pause(1);
            end
            aligned_imgs{kk,aa} = padarray(aligned_imgs{kk,aa}, ...
                                           max(size(template{aa}(:,:,1)) - ...
                                               size(aligned_imgs{kk,aa}(:,:,1)), ...
                                               0), 'post');
            aligned_imgs{kk,aa} = aligned_imgs{kk,aa}(1:size(template{aa},1), ...
                                                      1:size(template{aa},2),:);
            fprintf(1, '.');
        end
    end
        
    % All images are aligned. Compute the average image.
    left_avg = zeros(size(template{1}));
    right_avg = zeros(size(template{2}));
    num_null = [0 0];
    for kk = 1:size(aligned_imgs,1),        
        if numel(aligned_imgs{kk,1}) ~= 0,
            left_avg = left_avg + aligned_imgs{kk,1};
        else
            num_null(1) = num_null(1) + 1;
        end

        if size(aligned_imgs,2) > 1,
            if numel(aligned_imgs{kk,2}) ~= 0,
                right_avg = right_avg + aligned_imgs{kk,2};
            else
                num_null(2) = num_null(2) + 1;
            end
        end
    end
    left_avg = left_avg ./ (size(aligned_imgs,1) - num_null(1));
    right_avg = right_avg ./ (size(aligned_imgs,1) - num_null(2));
    fprintf(1, '.');
    
    % We have a biased mean image right now. Transform that biased
    % image to an unbiased one.
    template_sz = [size(template{1}, 1) size(template{1}, 2) ...
                   size(template{1}, 3); size(template{2}, 1) ...
                   size(template{2}, 2) size(template{2}, 3)];
    unbias_transforms = create_unbiasing_transforms(transforms, ...
                                                    template_sz);
    fprintf(1, '.');
    
    % Apply the deformable transformation first, then the affine
    % transformation 
    left_avg = my_img_warp_inv(left_avg, ...
                               unbias_transforms{1}.affine_warp.T, ...
                               unbias_transforms{1}.disp_field);
    
    if numel(right_avg) > 0,
        right_avg = my_img_warp_inv(right_avg, ...
                                    unbias_transforms{2}.affine_warp.T, ...
                                    unbias_transforms{2}.disp_field);
    end
        
    if strncmp(img_type, 'full', 4),        
        left_avg = time_warp_cont(left_avg, ...
                             unbias_transforms{1}.time_map);
        if numel(right_avg) > 0,
            right_avg = time_warp_cont(right_avg, ...
                                       unbias_transforms{2}.time_map);
        end
    end
    fprintf(1, '.');
    
    atlas{1} = left_avg;
    atlas{2} = right_avg;
    
    
    
% Invert and average the given transformations
function inv_avg_tforms = create_unbiasing_transforms(transforms, template_sizes)

    num_frames = template_sizes(:,3)';
    for aa = 1:size(transforms,2),

        max_sz = template_sizes(aa,1:2);        
        avg_disp_field = zeros([max_sz 2]);
        num_null = 0;
        
        % Now average and invert the deformation vector field.
        for ii = 1:size(transforms,1),
            if numel(transforms{ii,aa}.disp_field) ~= 0,
                tmp_warp = padarray(transforms{ii,aa}.disp_field, ...
                                    max(template_sizes(aa, 1:2) - ...
                                        size(transforms{ii,aa}.disp_field(:,:,1)), ...
                                        0), 'post');
                avg_disp_field = avg_disp_field + tmp_warp(1:template_sizes(aa,1), ...
                                                           1:template_sizes(aa,2), :);
            else
                num_null = num_null + 1;
            end
        end
        avg_disp_field = avg_disp_field ./ (size(transforms,1) - num_null);
        inv_avg_tforms{aa}.disp_field = ...
            invert_disp_field(avg_disp_field);
        
        % Next, average and invert the affine matrices.
        avg_affine_mat = zeros(3);
        for ii = 1:size(transforms,1),
            avg_affine_mat = avg_affine_mat + transforms{ii,aa}.affineMat;
        end
        avg_affine_mat = avg_affine_mat ./ (size(transforms,1) - num_null);
        inv_avg_tforms{aa}.affine_warp = invert(affine2d(avg_affine_mat));
        
        % Finally, average the dynamic time warp mapping
        %inv_avg_tforms{aa}.time_map = ...
        %    avg_inv_time_warp(transforms(:,aa));

    end