function reg_img = my_img_warp_inv(img, affineMat, disp_field)
% function reg_img = my_img_warp_inv(img, affineMat, disp_field)
%
% Apply the given spatial transformations to the given
% image. Interpolate the transformed image using sinc
% interpolation. If both a linear and deformable transformation are
% specified, concatenate the two transformations before applying
% them to the image. 
%
% Unlike my_img_warp.m, this function applies the non-linear
% deformation first, then the affine deformation.
%
% Arguments:
%      - img: The 2D image to transform.
%      - affineMat: The linear (rigid or affine) transformation to
%                   apply to the image.
%      - disp_field: The field of displacement vectors to apply to
%                    the image (optional).
%
% Returns:
%      - reg_img: The inputted image transformed by the given
%                 trasformation(s). 
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 25, 2016
%

    % convert the linear transformation to a displacement field.
    R_in = imref2d(size(img(:,:,1)));
    tform = affine2d(affineMat);
    [XWorldLimitsOut,YWorldLimitsOut] = outputLimits(tform, ...
                                                     R_in.XWorldLimits,R_in.YWorldLimits);
    %XWorldLimitsOut(1) = R_in.XWorldLimits(1);
    %YWorldLimitsOut(1) = R_in.YWorldLimits(1);
    R_out = images.spatialref.internal.snapWorldLimitsToSatisfyResolution([XWorldLimitsOut; YWorldLimitsOut],...
        [R_in.PixelExtentInWorldX, R_in.PixelExtentInWorldY]);
        
    linear_disps = get_linear_disp_field(R_in, tform, R_out);
    
    D = concatenate_disps(disp_field, flip(linear_disps, 3));
    
    reg_img = warp_image(img, D);

    % crop the image to bounding box, then pad slightly
    seg = threshold_scan(reg_img);
    [rr cc] = find(max(seg, [], 3));
    reg_img = reg_img(min(rr):max(rr), min(cc):max(cc), :);
    reg_img = padarray(reg_img, [3 3 0], 'both');
    
    %imagesc(max(reg_img, [], 3)); axis image; colorbar;
    
    return;
    
    [inXX, inYY] = meshgrid(1:size(disp_field, 2), 1:size(disp_field, 1));
    outXX = inXX + disp_field(:,:,1);
    outYY = inYY + disp_field(:,:,2);
        
    % Concatenate linear displacement field with given disp_field
    % if provided.
    [sx sy] = meshgrid(1:size(disp_field,2), 1:size(disp_field,1));
    [nx ny] = meshgrid(1:size(linear_disps,2), 1:size(linear_disps,1));

    for aa = size(disp_field,3):-1:1,
        F = scatteredInterpolant(nx(:), ny(:), ...
                                 reshape(linear_disps(:,:,aa), ...
                                         [numel(nx) 1]));
        affine_disps(:,:,aa) = F(sx,sy);

    end
    D = disp_field + affine_disps;

    % Get the final image size.
    invD = invert_disp_field(D);
    finalXX = inXX + invD(:,:,1);
    finalYY = inYY + invD(:,:,2);

    out_bounds = [floor(min(finalYY(:))) ceil(max(finalYY(:))); ...
                  floor(min(finalXX(:))) ceil(max(finalXX(:)))];
    
    % Pad the image and deformation so that we get an image that is
    % the same size as the requested output image size.
    D = pad_matrix(D, out_bounds, true);
    img = pad_matrix(img, out_bounds, false);

    % Apply deformation and return.
    if ndims(img) == 3,
        D = reshape(D, [size(D,1) size(D,2) 1 size(D,3)]);
        D = D(:,:,ones(1,size(img,3)),:);
        D(:,:,:,3) = zeros(size(D(:,:,:,1)));
    end

    reg_img = real(imwarp(img, D, 'cubic'));
    reg_img = max(0, reg_img);
    return;
    
    % ALTERNATIVE: sinc interpolation 
    %
    % Very slow and has ringing artifacts!
    %
    % We now have the full deformation in D. Apply it to the
    % original image and use 2D sinc interpolation to get the
    % result.
    [imgXX, imgYY] = meshgrid(1:ceil(max(finalXX(:))), ...
                              1:ceil(max(finalYY(:))));
    reg_img = interp_frame(img, finalXX, finalYY, imgXX, imgYY);


% Concatenate diffeomorphisms
function D = concatenate_disps(disps1, disps2);

    % check for empty deformation field
    if numel(disps1) == 0,
        D = disps2;
        return;
    end
    
    % check for differences in size and pad the smaller
    % displacement field
    if sum(abs(size(disps1) - size(disps2))),
        new_sz = max(size(disps1), size(disps2));
        pad_sz1 = new_sz - size(disps1);
        pad_sz2 = new_sz - size(disps2);
        
        new_disps1 = zeros(new_sz);
        new_disps1(1:size(disps1,1), 1:size(disps1,2), :) = disps1;
        new_disps2 = zeros(new_sz);
        new_disps2(1:size(disps2,1), 1:size(disps2,2), :) = disps2;
        %new_disps1 = padarray(disps1, [pad_sz1 0], 'post');
        %new_disps2 = padarray(disps2, [pad_sz2 0], 'post');
        
        disps1 = new_disps1(:,:,1:2);
        disps2 = new_disps2(:,:,1:2);
    end
    
    % interpolate the displacements for disps2 to the displaced
    % pixel coordinates from disps1
    interp_disps1 = zeros(size(disps2));
    interp_disps1(:,:,1) = warp_image(disps1(:,:,1), disps2);
    interp_disps1(:,:,2) = warp_image(disps1(:,:,2), disps2);
    %interp_disps2 = interp_disps2(1:size(disps1,1), 1:size(disps1,2),:);
    
    % combine displacements
    D = disps2;
    D(:,:,1) = D(:,:,1) + interp_disps1(:,:,1);
    D(:,:,2) = D(:,:,2) + interp_disps1(:,:,2);

    
% Warp the given image with the given deformation
function new_image = warp_image(old_image, D)

    % if the image is bigger than the displacement field, pad the
    % displacement field.
    % if nnz(size(old_image(:,:,1)) - size(D(:,:,1)) > 0),
    %     pad_sz = max(0, size(old_image(:,:,1)) - size(D(:,:,1)));
    %     D = padarray(D, [pad_sz 0], 'replicate', 'post');
    % end
        
    %D = D(1:size(old_image,1), 1:size(old_image,2), :);

    % interpolate new image at given positions
    if ndims(old_image) > 2,
        % duplicate the 2D deformation over the 3rd dimension
        F = zeros([size(D,1) size(D,2) size(old_image,3), 2]);
        F(:,:,:,1) = D(:,:,ones(size(old_image,3),1));
        F(:,:,:,2) = D(:,:,2*ones(size(old_image,3),1));
        
        new_image = zeros(size(F(:,:,:,1)));
        
        % get pixel coordinates
        [ox, oy, oz] = meshgrid(0:size(old_image,2)-1, 0:size(old_image,1)-1, ...
                                0:size(old_image,3)-1);
        [px, py, pz] = meshgrid(0:size(new_image,2)-1, 0:size(new_image,1)-1, ...
                                0:size(new_image,3)-1);
        new_x = px + F(:,:,:,2);
        new_y = py + F(:,:,:,1);
        
        new_image = interp3(ox, oy, oz, old_image, new_x, new_y, pz, ...
                            'linear', 0);
    else
        % get pixel coordinates
        new_image = zeros(size(D(:,:,1)));
        
        [ox, oy] = meshgrid(0:size(old_image,2)-1, 0:size(old_image,1)-1);
        [px, py] = meshgrid(0:size(new_image,2)-1, 0:size(new_image,1)-1);
        new_x = px + D(:,:,2);
        new_y = py + D(:,:,1);

        % interpolate new image at given positions
        new_image = interp2(ox, oy, old_image, new_x, new_y, 'cubic', 0);
    end

    
% Pad image and/or deformation to match image output size
function pad_img = pad_matrix(img, bounds_out, fill_opt)

    imSz = size(img)';
    pre_pad_size = max(0, 1 - bounds_out(:,1));
    post_pad_size = max(0, bounds_out(:,2) - imSz(1:2));
    
    % replicate the final rows/colums if asked
    if fill_opt,
        pad_img = padarray(img, pre_pad_size, 'pre', 'replicate');
        pad_img = padarray(img, post_pad_size, ...
                           'post', 'replicate');
    else
        pad_img = padarray(img, pre_pad_size, 'pre');
        pad_img = padarray(img, post_pad_size, 'post');
    end
    
% Interplolate a 2D image frame using a sync function
function new_img = interp_frame(img, inXX, inYY, outXX, outYY)
        
    new_img_vec = sinc_interp(img(:), inXX(:), inYY(:), outXX(:), outYY(:));
    new_img = reshape(new_img_vec, size(outXX));
    
% "Ideal" 2D sinc interpolation.
function y = sinc_interp(img, oldXX, oldYY, newXX, newYY)

    %step = round(length(newXX) / 40);
    %fprintf(1, 'Interpolating...');
    %oldXX = oldXX';
    %oldYY = oldYY';
    img = img';
    for ii = length(newXX):-1:1,
        MY = newXX(ii) - oldXX;
        MX = newYY(ii) - oldYY;
        
        y(ii) = img * (sinc(MX) .* sinc(MY));
        %if mod(ii,step) == 0,
        %    fprintf(1, '.');
        %end
    end
    %fprintf(1, 'Done!\n');
            
% Converts the affine/rigid transformation to a displacement vector field.
function [disp_field, inv_disps] = get_linear_disp_field(Rin, tform, Rout)

    % Form plaid grid of intrinsic points in output image.
    [dstXIntrinsic,dstYIntrinsic] = meshgrid(1:Rout.ImageSize(2),1:Rout.ImageSize(1));

    % Define affine transformation that maps from intrinsic system of
    % output image to world system of output image.
    Sx = Rout.PixelExtentInWorldX;
    Sy = Rout.PixelExtentInWorldY;
    Tx = Rout.XWorldLimits(1)-Rout.PixelExtentInWorldX*(Rout.XIntrinsicLimits(1));
    Ty = Rout.YWorldLimits(1)-Rout.PixelExtentInWorldY*(Rout.YIntrinsicLimits(1));
    tIntrinsictoWorldOutput = [Sx 0 0; 0 Sy 0; Tx Ty 1];

    % Define affine transformation that maps from world system of
    % input image to intrinsic system of input image.
    Sx = 1/Rin.PixelExtentInWorldX;
    Sy = 1/Rin.PixelExtentInWorldY;
    Tx = (Rin.XIntrinsicLimits(1))-1/Rin.PixelExtentInWorldX*Rin.XWorldLimits(1);
    Ty = (Rin.YIntrinsicLimits(1))-1/Rin.PixelExtentInWorldY*Rin.YWorldLimits(1);
    tWorldToIntrinsicInput = [Sx 0 0; 0 Sy 0; Tx Ty 1];

    % Form transformation to go from output intrinsic to input intrinsic
    tComp = tIntrinsictoWorldOutput / tform.T * tWorldToIntrinsicInput;

    % Find the transform that takes from input intrinsic to output
    % intrinsic. NOTE: if using sinc interpolation, invert this transformation.
    tformComposite = (affine2d(tComp(1:3,1:2)));
    
    % Apply transformation to meshgrid to get pixel displacements.
    imSz = [max(Rin.ImageSize(2), Rout.ImageSize(2)), ...
            max(Rin.ImageSize(1), Rout.ImageSize(1))];
    [srcXIntrinsic, srcYIntrinsic] = meshgrid(1:imSz(1),1:imSz(2));
    [dstXIntrinsic, dstYIntrinsic] = tformComposite.transformPointsForward( ...
        srcXIntrinsic, srcYIntrinsic);

    % Create deformation.
    disp_field = zeros([size(dstXIntrinsic) 2]);
    disp_field(:,:,1) = dstXIntrinsic - srcXIntrinsic;
    disp_field(:,:,2) = dstYIntrinsic - srcYIntrinsic;
    
    % Repeat for the inverse
    imSz = [max(Rin.ImageSize(2), Rout.ImageSize(2)), ...
            max(Rin.ImageSize(1), Rout.ImageSize(1))];
    [dstXIntrinsic, dstYIntrinsic] = meshgrid(1:imSz(1),1:imSz(2));
    tformInverse = invert(tformComposite);
    [srcXIntrinsic, srcYIntrinsic] = tformInverse.transformPointsForward( ...
        dstXIntrinsic, dstYIntrinsic);

    % Create deformation.
    inv_disps = zeros([size(srcXIntrinsic) 2]);
    inv_disps(:,:,1) = srcXIntrinsic - dstXIntrinsic;
    inv_disps(:,:,2) = srcYIntrinsic - dstYIntrinsic;
