function [pred_img, snr_img] = predict_img(weight_imgs, covars, err_img)
% function [pred_img, snr_img] = predict_img(weight_imgs, covars, err_img)
%
% Predict the plantar pressure image for a subject with the given
% covariate settings using the linear regression model.
%
% Arguments:
%     - weight_imgs: The linear regression coefficients from the
%                    statistical model.
%     - covars: The covariate values for the subject whose plantar
%               pressure scan we want to predict.
%     - err_img: The model's standard error image.
%
% Returns:
%     - pred_img: The predicted plantar pressure image for the
%                 subject with the provided covariate values.
%     - snr_img: The signal to noise ratio image for the model
%                prediction. 
%
% Written by Brian Booth <brian.booth@uantwerpen.be>, September 30, 2016
%

    pred_img = zeros(size(err_img));

    if ndims(weight_imgs) == 3,
        pred_img = weight_imgs(:,:,1);
        for ii = 2:size(weight_imgs, 3),
            pred_img = pred_img + covars(ii-1) * ...
                weight_imgs(:,:,ii);
        end
    else
        pred_img = weight_imgs(:,:,:,1);
        for ii = 2:size(weight_imgs, 4),
            pred_img = pred_img + covars(ii-1) * ...
                weight_imgs(:,:,:,ii);
        end
    end

    pred_img = max(0, pred_img);
    
    snr_img = pred_img ./ max(eps, err_img);