function new_set = sanitize_image_set(img_set, normalize)
% function new_set = sanitize_image_set(img_set, normalize)
%
% Interpolate each 2D+time image volume to account for the sensor
% dimensions. Also check for any feet that are upside-down and flip
% them so that the toes are consistantly at the top of the image.
%
% Arguments:
%       - img_set: A set of plantar pressure images loaded with the
%                  load_ppressure_scan script.
%
% Returns:
%       - new_set: The same plantar pressure image set, but with each
%                  image properly interpolated and oriented.
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 4, 2016
% 

  if nargin == 1,
      normalize = 0;
  end

  if isempty(img_set),
      new_set = [];
      return;
  end
  
  % Interpolate the image to account for the sensor dimensions
  s_dim = img_set.hdr.pixel_size_mm;
  new_set = img_set;
  new_set.hdr.pixel_size_mm = [1 1];
  bad_trials = zeros(0,1);
  
  for ii = 1:numel(img_set.imgs),
  
      img = img_set.imgs(ii).img;

      if isempty(img),
          new_set.imgs(ii) = img_set.imgs(ii);
          bad_trials = [bad_trials; ii];
          continue;
      end
      
      [ox, oy, oz] = meshgrid(0:size(img,2)-1, 0:size(img,1)-1, 1:size(img,3));
      ox = s_dim(2) * ox;
      oy = s_dim(1) * oy;
      
      [nx, ny, nz] = meshgrid(0:3:round(max(ox(:))), 0:3:round(max(oy(:))), ...
                              1:size(img,3));
      
      new_set.imgs(ii).img = interp3(ox, oy, oz, img, nx, ny, nz, 'linear', 0);
      
      %for jj = size(img,3):-1:1,
      %    img2(:,:,jj) = interp_frame(img(:,:,jj), s_dim, [1 1]);
      %end
      
      % Move from N/cm2 to kPa
      % new_set.imgs(ii).img = 10 * img2;
      % new_set.imgs(ii).img = img2;
            
      if normalize,
          % Normalize by total mean pressure
          meanIM = scalarize_foot_pressure_img(new_set.imgs(ii).img, ...
                                              'mean');
          meanIM = meanIM ./ 1000;
          % durationIM = scalarize_foot_pressure_img(new_set.imgs(ii).img, ...
          %                                          'duration');      
          % sumIM = sumIM ./ max(eps, durationIM);
          % sumIM = sumIM .* (durationIM > 0.05);
          
          new_set.imgs(ii).img = 10 * new_set.imgs(ii).img ./ sum(meanIM(:));
      end
      
      % Handle numerical stability
      % new_set.imgs(ii).img = new_set.imgs(ii).img;
      
      new_set.imgs(ii).bounding_box = round(new_set.imgs(ii).bounding_box ...
          .* [s_dim s_dim(2) s_dim(1)]);

      %clearvars a_frame img2;
  end
  
  % Remove missing trials
  new_set.imgs(bad_trials) = [];
  
  % Identify sideways feet and rotate them
  for ii = 1:numel(new_set.imgs),      
      if new_set.imgs(ii).bounding_box(3) > ...
              new_set.imgs(ii).bounding_box(4),
          
          new_set.imgs(ii).img = permute(new_set.imgs(ii).img, ...
                                         [2 1 3]);
          new_set.imgs(ii).bounding_box(3:4) = ...
              flip(new_set.imgs(ii).bounding_box(3:4));
      end

  end
    
  % Identify upside-down feet and flip them
  for ii = 1:numel(new_set.imgs),
  
      tmpIM = new_set.imgs(ii).img;

      [cop, vop] = cop_vop(tmpIM);
      
      % Assuming a heel-to-toe step, the directions of the
      % centroids should all move upward in the image space.
      step_direction = mean(diff(cop));
      step_direction = step_direction(2);
      step_speed = mean(vop(:,3));
      
      %view_one_scan(new_set.imgs(ii).img);
      
      % If the heel-to-toe step is in the wrong direction in the image
      % then flip the image.
      if step_direction < 0 & step_speed > 0.016,
          new_set.imgs(ii).img = flipdim(new_set.imgs(ii).img, 1);
          new_set.imgs(ii).img = flipdim(new_set.imgs(ii).img, 2);
      end
      
      % Static image: can't use COP. Use foot width as the forefoot
      % is wider than the heel.
      if step_speed < 0.016,
          segIM = threshold_scan(tmpIM);
          foot_width = sum(sum(segIM, 3), 2);
          [~, max_loc] = max(foot_width);
                    
          if max_loc > numel(foot_width) / 2,
              new_set.imgs(ii).img = flipdim(new_set.imgs(ii).img, 1);
              new_set.imgs(ii).img = flipdim(new_set.imgs(ii).img, 2);
          end
          %new_set.imgs(ii).img = flipdim(new_set.imgs(ii).img, 2);
          
      end
      
      % view_one_scan(new_set.imgs(ii).img);
      % drawnow;
      % pause;
      
  end
  
  % Filp left-right of each image so that the feet show up
  % correctly in MATLAB.
  for ii = 1:numel(new_set.imgs),
      new_set.imgs(ii).img = flipdim(new_set.imgs(ii).img, 2);
  end

  % Find trials that do not contain the whole foot by thresholding
  % each trial and checking the image perimeter for sensors that are
  % on. Order the trials from least perimeter sensors on to most.
  new_set = reorder_set(new_set);
  
  % Interpolate the time dimension if the image is not acquired at 200Hz.
  % for aa = 1:2,
  %     if new_set.hdr.scan_speed(aa) ~= 200,
  %         for bb = 1:numel(new_set.imgs),
  %             if (aa == 1 & strncmp('left', ...
  %                                   new_set.imgs(bb).foot_side, 4)) ...
  %                     | (aa == 2 & strncmp('right', ...
  %                                          new_set.imgs(bb).foot_side, 5)),
  %                 new_set.imgs(bb).img = ...
  %                     interp_time(new_set.imgs(bb).img, ...
  %                                 new_set.hdr.scan_speed(aa), 200);
  %                 new_set.imgs(bb).num_frames = ...
  %                     size(new_set.imgs(bb).img,3);
  %             end
              
  %         end
  %     end
  % end
  
% Reorder trials by the amount of foot that is along the image
% perimeter (from least to most)
function img_set = reorder_set(img_set)
    
    foot_side = zeros(1, numel(img_set.imgs));
    foot_sz = zeros(1, numel(img_set.imgs));
    
    % compute the perimeter overlap
    for ii = 1:numel(img_set.imgs),
        if strncmp(img_set.imgs(ii).foot_side, 'left', 4),
            foot_side(ii) = 1;
        else
            foot_side(ii) = 2;
        end
        
        mask_img = threshold_scan(img_set.imgs(ii).img);
        mask_img = max(mask_img, [], 3);
        
        foot_sz(ii) = nnz(mask_img);
    end
    
    % do the reordering
    idl = find(foot_side == 1);
    [~, idx] = sort(foot_sz(idl), 'descend');
    img_set.imgs(idl) = img_set.imgs(idl(idx));
    
    idr = find(foot_side == 2);
    [~, idx] = sort(foot_sz(idr), 'descend');
    img_set.imgs(idr) = img_set.imgs(idr(idx));
    
    
% Interpolate an image along the time dimension in order to
% standardize the imaging frequency
function new_img = interp_time(img, old_freq, new_freq)
    
    img = permute(img, [3 1 2]);
    X = 0:(1/old_freq):((size(img,1)-1) / old_freq);
    XX = 0:(1/new_freq):max(X(:));
    new_img = interp1(X, img, XX, 'pchip', 'extrap');
    new_img = permute(new_img, [2 3 1]);
  
% Interplolate a 2D image frame using a sync function
function new_frame = interp_frame(img, sensor_sz, new_sz)
    
    % We have a grid of sensors so we can linearly separate the
    % sync interpolations along both dimensions.
    for aa = 1:2,
        s = sensor_sz(aa):sensor_sz(aa):size(img,aa)*sensor_sz(aa);
        u = sensor_sz(aa):size(img,aa)*sensor_sz(aa);
        imSz = size(img);
        imSz(aa) = numel(u);
        new_frame = zeros(imSz);
        if aa == 2,
            for ii = 1:size(img,1),
                new_frame(ii,:) = sinc_interp(img(ii,:),s,u);
            end
        else
            for ii = 1:size(img,2),
                new_frame(:,ii) = sinc_interp(img(:,ii)',s,u)';
            end
        end
        img = new_frame;
    end
    
% From http://phaseportrait.blogspot.com/2008/06/sinc-interpolation-in-matlab.html
% Ideally "resamples" x vector from s to u by sinc interpolation
function y = sinc_interp(x,s,u)
    % Interpolates x sampled sampled at "s" instants
    % Output y is sampled at "u" instants ("u" for "upsampled")
    % (EXPECTS x, s, and u to be ROW VECTORS!!)

    % Find the period of the undersampled signal
    T = s(2)-s(1);
    
    % When generating this matrix, remember that "s" and "u" are
    % passed as ROW vectors and "y" is expected to also be a ROW
    % vector. If everything were column vectors, we'd do.
    %
    % sincM = repmat( u, 1, length(s) ) - repmat( s', length(u), 1 );
    %
    % So that the matrix would be longer than it is wide.
    % Here, we generate the transpose of that matrix.
    sincM = repmat( u, length(s), 1 ) - repmat( s', 1, length(u) );

    % Equivalent to column vector math:
    % y = sinc( sincM'/T )*x';
    %whos
    %size(sinc( sincM/T ))
    y = x*sinc( sincM/T );
