function [reg_img, affineMat, disp_field] = scalar_img_register(fixed_img, ...
                                                  moving_img, reg_type)
% function [reg_img, affineMat, disp_field] = scalar_img_register(fixed_img, ...
%                                                 moving_img, reg_type)
%
% Take a pair of peak pressure maps from two different people and
% align them to each other. Use a combination of affine
% registration on the raw peak pressure map (using mutual
% information as a metric), then a deformable Demons registration
% on the foot segmentations to get the final alignment. 
% Alternatively, you can also settle for an affine alignment.
%
% Arguments:
%       - fixed_img: The fixed image that we will register to.
%       - moving_img: The moving image that will be registered.
%       - reg_type: The registration transform type used in the
%                   registration. Options are 'affine' or 'deformable'.
%
% Returns:
%       - reg_img: The registered moving image
%       - affineMat: 
%       - disp_field: The displacement field representing the
%                     deformation, or an empty matrix if 'affine'
%                     registration type selected.
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 6, 2016
%

    if ndims(fixed_img) == 3,
        old_fixed = fixed_img;
        old_moving = moving_img;
        fixed_img = sum(fixed_img, 3);
        moving_img = sum(moving_img, 3);
    else
        old_moving = [];
    end

    % Perform the affine registration
    if strncmp(lower(reg_type), 'affine', 6),
        %affineMat = fourier_affine_register(fixed_img, ...
        %                                    moving_img);
        affineMat = linear_register2(fixed_img, moving_img, ...
                                     'affine');
        %affineMat = foot_system_rigid_register(fixed_img, moving_img, ...
        %                                       'affine');
        %[~, affineMat] = pairwise_affine_register(fixed_img, ... 
        %                                          moving_img);
        disp_field = [];
    elseif strncmp(lower(reg_type), 'rigid', 5),
        %affineMat = fourier_affine_register(fixed_img, ...
        %                                    moving_img);
        affineMat = linear_register2(fixed_img, moving_img, ...
                                     'rigid');
        %affineMat = foot_system_rigid_register(fixed_img, moving_img, ...
        %                                       'rigid');
        %[~, affineMat] = pairwise_rigid_register(fixed_img, ... 
        %                                         moving_img);
        disp_field = [];
    elseif strncmp(lower(reg_type), 'deformable', 10),
        %affineMat = fourier_affine_register(fixed_img, ...
        %                                    moving_img);
        %affineMat = foot_system_rigid_register(fixed_img, moving_img, ...
        %                                       'affine');
        affineMat = linear_register2(fixed_img, moving_img, ...
                                     'affine');
        %[~, affineMat] = pairwise_affine_register(fixed_img, ... 
        %                                          moving_img);

        affineIM = my_img_warp(moving_img, affineMat, [], ...
                               size(fixed_img));

        %imshowpair(affineIM, fixed_img);
        %drawnow;
        % % disp('foo');
        %pause;
        
        % Create the foot segmentations
        fixed_seg = double(threshold_scan(fixed_img));
        affine_seg = double(threshold_scan(affineIM));
        
        % Perform the deformable registration on the foot segmentations
        [reg_seg, disp_field] = pairwise_deform_register(fixed_seg, ...
                                                         affine_seg); 
    else
        error('Unrecognized transfomation type!');
    end

    if numel(old_moving) > 0,
        fixed_img = old_fixed;
        moving_img = old_moving;
    end
    
    reg_img = my_img_warp(moving_img, affineMat, disp_field, ...
                          size(fixed_img));
    
    % subplot(1,3,1); imagesc(moving_img); axis image off; colormap hot
    % subplot(1,3,2); imagesc(fixed_img); axis image off; colormap hot
    % subplot(1,3,3); imshowpair(reg_img, fixed_img);
    % drawnow;
    % pause;

    
    % reg_img = zeros(size(fixed_img));
    % reg_img(1:min(size(fixed_img,1), size(tmp_img,1)), ...
    %         1:min(size(fixed_img,2), size(tmp_img,2)), :) = ...
    %     tmp_img(1:min(size(fixed_img,1), size(tmp_img,1)), ...
    %             1:min(size(fixed_img,2), size(tmp_img,2)), :);

