function scalar_img = scalarize_foot_pressure_img(img, scalar_type, dt)
% function scalar_img = scalarize_foot_pressure_img(img, scalar_type, dt)
%
% Generate a 2D image of the foot from a 2D+time plantar pressure
% image. Summarize the time dimension using the given choice of
% statistical measure.
%
% Arguments:
%      - img: The 2D+time plantar pressure image
%      - scalar_type: The choice of statistic used to summarize the
%                     time dimension. Options include:
%                      * 'peak': Peak/Max pressure image
%                      * 'peak_time': The time (frame number)
%                         corresponding to the peak pressure
%                         measurement. 
%                      * 'mean': Mean pressure image
%                      * 'median': Median pressure image
%                      * 'sum': Pressure-Time integral image
%                      * 'var': Pressure variance image
%                      * 'std': Pressure standard deviation image
%                      * 'duration': The number of frames for which
%                                    the foot is touching the
%                                    ground.
%                      * 'cop_dist': Distance to center of pressure
%                                    trajectory 
%                      * 'touch_down': Time at which a part of the
%                                      foot hits the ground.
%                      * 'lift_off': Time at which a part of the
%                                    foot leaves contact with the
%                                    ground. 
%                      * 'entropy': Pressure entropy image
%                      * 'kurtosis': Pressure kurtosis image
%                      * 'skewness': Pressure skewness image
%                      * 'fft_mag': The power spectrum for the
%                         Fourier transformed time series 
%                         at each pixel.
%                      * 'fft_phase': The phase spectrum for the
%                         Fourier transformed time series 
%                         at each pixel.
%     - dt: The change in time between two image frames. Required
%           for measurements of peak_time, duration, lift_off, and
%           touch_down. 
%
% Returns:
%      - scalar_img: The scalarized plantar pressure image
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 5th, 2016
%
    
    if nargin == 2,
        dt = 5;
    end

    img(img < 0) = 0;
    
    switch lower(scalar_type),
    
      case 'peak',
        scalar_img = max(img, [], 3);
      case 'peak_time',
        [~, scalar_img] = max(img, [], 3);
        scalar_img = scalar_img * dt;
      case 'peak_rate',
        diff_img = diff(img, 1, 3) / dt;
        scalar_img = max(diff_img, [], 3);
      case 'mean',
        mask_img = threshold_scan(img);
        img(mask_img == 0) = NaN;
        scalar_img = mean(img, 3, 'omitnan');
        scalar_img(isnan(scalar_img)) = 0;
      case 'mean_rate',
        diff_img = diff(img, 1, 3) / dt;
        [~, peak_time] = max(img, [], 3);
        for ii = 1:size(peak_time,1),
            for jj = 1:size(peak_time,2),
                idz = peak_time(ii,jj)+1;
                diff_img(ii,jj,idz:end) = NaN;
            end
        end
        scalar_img = mean(diff_img, 3, 'omitnan');
        scalar_img(isnan(scalar_img)) = 0;
      case 'median'
        mask_img = threshold_scan(img);
        img(mask_img == 0) = NaN;
        scalar_img = median(img, 3, 'omitnan');
        scalar_img(isnan(scalar_img)) = 0;
      case 'sum'
        scalar_img = sum(img, 3);
      case 'var',
        mask_img = threshold_scan(img);
        img(mask_img == 0) = NaN;
        scalar_img = var(img, 0, 3, 'omitnan');
        scalar_img(isnan(scalar_img)) = 0;
      case 'std',
        mask_img = threshold_scan(img);
        img(mask_img == 0) = NaN;
        scalar_img = std(img, 0, 3, 'omitnan');
        scalar_img(isnan(scalar_img)) = 0;
      case 'duration'
        seg_img = threshold_scan(img);
        scalar_img = sum(seg_img, 3);
        scalar_img = scalar_img ./ size(img,3) * 100;
        % case 'cop_dist'
        % scalar_img = double(center_of_pressure_img(img));
      case 'touch_down'
        seg_img = threshold_scan(img);
        [mask_img, scalar_img] = max(seg_img, [], 3);
        scalar_img = scalar_img ./ size(img,3) * 100;
        scalar_img = scalar_img .* mask_img;
      case 'lift_off'
        seg_img = threshold_scan(img);
        seg_img = flip(seg_img, 3);
        [mask_img, scalar_img] = max(seg_img, [], 3);
        scalar_img = size(img,3) - scalar_img + 1;
        scalar_img = scalar_img ./ size(img,3) * 100;
        scalar_img = scalar_img .* mask_img;
      % case 'entropy',
      %   % Decide on the histogram binning. Use one bin for roughly every
      %   % ten samples. Have bins linearly spaced over the domain.
      %   imgMax = max(img(:));
      %   num_bins = floor(size(img,3) / 10);
      %   edges = linspace(0, imgMax, num_bins);
      %   % Compute entropy across time dimension by using mat2cell
      %   tmpCell = mat2cell(img, ones(1, size(img,1)), ones(1, size(img,2)), ...
      %                      size(img,3));
      %   scalar_img = cellfun(@(x) matrix_entropy(x, edges), tmpCell);
      % case 'kurtosis',
      %   tmpCell = mat2cell(img, ones(1, size(img,1)), ones(1, size(img,2)), ...
      %                      size(img,3));
      %   scalar_img = cellfun(@kurtosis, tmpCell);        
      % case 'skewness',
      %   tmpCell = mat2cell(img, ones(1, size(img,1)), ones(1, size(img,2)), ...
      %                      size(img,3));
      %   scalar_img = cellfun(@skewness, tmpCell);   
      % case 'fft_mag'
      %   if mod(size(img,3), 2) == 1,
      %       img(:,:,end+1) = zeros(size(img(:,:,end)));
      %   end
      %   Y = abs(fft(img, [], 3));
      %   scalar_img = Y(:,:,1:32);
      % case 'fft_phase'
      %   if mod(size(img,3), 2) == 1,
      %       img(:,:,end+1) = zeros(size(img(:,:,end)));
      %   end
      %   Y = fft(img, [], 3);
      %   scalar_img = sunwrap(Y(:,:,1:32));
      otherwise
        error('Unexpected scalar image type!');        
    end
    
    scalar_img(isnan(scalar_img)) = min(scalar_img(:));