function [reg_img, transforms, reg_mask] = spatiotemporal_register( ...
    fixed_img, moving_img, reg_type)
% function [reg_img, tramsforms, reg_mask] = spatiotemporal_register( ...
%   fixed_img, moving_img, reg_type)
%
% Register one 2D+time planter pressure image to another. Use
% either affine or deformable registration to align spatially
% first, the register across time using dynaming time warping.
%
% Arguments:
%       - fixed_img: The fixed image that we will register to.
%       - moving_img: The moving image that will be registered.
%       - reg_type: The registration transform type used in the sptial
%                   registration. Options are 'affine' or 'deformable'.
%
% Returns:
%       - reg_img: The registered moving image.
%       - transforms: A structure containing the following
%                     transformation-based fields:
%                      * affineMat: The affine transformation matrix.
%                      * disp_field: The displacement field 
%                                    representing the deformable
%                                    sptial transformation.
%                      * time_map: A vector mapping containing the
%                                  moving image to fixed image
%                                  frame correspondances.
%
% Written by Brian Booth <Brian.Booth@uantwerpen.be>, January 6, 2016
%

    % Perform the spatial registration on the peak pressure maps
    template = scalarize_foot_pressure_img(fixed_img, 'peak');
    to_move_img = scalarize_foot_pressure_img(moving_img, 'peak');

    [tmpIM, affineMat, def_field] = scalar_img_register(template, ...
                                                      to_move_img, ...
                                                      reg_type); 
    % Apply the deformation to each frame
    spatial_reg_img = my_img_warp(moving_img, affineMat, def_field, ...
                                  size(fixed_img));
    mask_img = ones(size(moving_img));
    spatial_reg_mask = my_img_warp(mask_img, affineMat, def_field, ...
                                   size(fixed_img));
    
    % spatial_reg_img = padarray(spatial_reg_img, ...
    %                            max(0, size(fixed_img(:,:,1)) - ...
    %                                size(spatial_reg_img(:,:,1))), ...
    %                            'post');
    % spatial_reg_img = spatial_reg_img(1:size(fixed_img,1), ...
    %                                   1:size(fixed_img,2), :);
    
    % Now, perform the time dimension registration using dynamic
    % time warping.
    time_map = dynamic_time_warping(fixed_img, spatial_reg_img);
    reg_img = time_warp_cont(spatial_reg_img, time_map);
    reg_img(reg_img < 0) = 0;
    reg_img = real(reg_img);
    
    reg_mask = time_warp_cont(spatial_reg_mask, time_map);
    
    % Use continuous dynamic time warping
    %[reg_img, time_map] = continuous_DTW(fixed_img, spatial_reg_img);
    
    % Gather all transformations
    transforms.affineMat = affineMat;
    transforms.disp_field = def_field;
    transforms.time_map = time_map;
    