function [image_set, demographics] = load_cad_walk_data(data_folder)
% function image_set = load_cad_walk_data(data_folder)
%
% Open the CAD WALK plantar pressure dataset located in the given
% data folder.
%
% Inputs:
%    - data_folder: Path to the data folder containing the CAD WALK
%                   plantar pressure measurements.
%
% Outputs:
%    - image_set: An Nx1 cell array of the peak pressure images
%                 that appear in the database following the
%                 preprocessing described in the PAPPI paper.
%    - demographics: a 5xN matrix containing the demographic
%                    information associated with each of the N peak
%                    pressure images. Specifically, the following
%                    demographics are provided in the order below:
%                     * age (in years)
%                     * sex (0 for female, 1 for male)
%                     * weight (in kilograms)
%                     * height (in centimeters)
%                     * shoe size (in the EU scale)
%
% Written by Brian Booth <brian.booth@uantwerpen.be>
%

  if ~exist('load_nifti_set'),
      p = mfilename('fullpath');
      [fdir,~,~] = fileparts(p);
      addpath([fdir, filesep, 'helper_functions']);
  end

  % Get a list of image directories
  files = dir(data_folder);
  dirFlags = [files.isdir];
  imgDirs = files(dirFlags);
  
  % Load and preprocess images
  image_set = cell(numel(imgDirs), 2);
  N = 0;
  
  fprintf('Preprocessing CAD WALK Data...');
  for ii = 1:numel(imgDirs),
     
      if ~(strncmp(imgDirs(ii).name, 'C', 1) | ...
           strncmp(imgDirs(ii).name, 'H', 1)),
          continue;
      end

      N = N+1;
      img_path = [imgDirs(ii).folder, filesep, imgDirs(ii).name];
      img_set = load_nifti_set(img_path);
      new_set = sanitize_image_set(img_set,1);
      [left_img, right_img] = intrasubject_avg_img(new_set);
      image_set{N,1} = scalarize_foot_pressure_img(left_img, 'peak');
      image_set{N,2} = scalarize_foot_pressure_img(right_img, 'peak');
      % Filp right foot to make it a left foot
      image_set{N,2} = flip(image_set{N,2}, 2);
      fprintf('.');
      
  end
  
  image_set = image_set(1:N, :);
  image_set = image_set(:);
  
  % Load the demographic information
  demographics = read_demographics([data_folder, filesep, ...
                      'metadata.xlsx']);
  fprintf('Done!\n');
  
  
% Function to read and properly format the deomgraphic data from a
% CAD WALK excel file
function M = read_demographics(filename)

  T = readtable(filename);
  T = T(2:end, 3:7);
  
  xx = T{:,4};
  yy = zeros(size(xx));
  for ii = 1:size(T,1),
      if strlength(xx{ii}) == 4,
          yy(ii) = 1;
      else
          yy(ii) = 0;
      end
  end
  
  M = table2array(T(:, [1:3,5]));
  M(:,end+1) = yy;
  
  M = M(:,[3 5 2 1 4]);
  M = [M; M]';
  